@@ -3,16 +3,16 @@ from typing import TYPE_CHECKING
 from applitools.common import logger
 from applitools.common.utils import argument_guard, image_utils
 from applitools.core.locators import (
-    VisualLocatorsProvider,
-    VisualLocatorSettings,
     LOCATORS_TYPE,
     VisualLocatorsData,
+    VisualLocatorSettings,
+    VisualLocatorsProvider,
 )
 
 if TYPE_CHECKING:
     from applitools.common.utils.custom_types import AnyWebDriver
-    from applitools.core.debug import DebugScreenshotsProvider
     from applitools.core import ServerConnector
+    from applitools.core.debug import DebugScreenshotsProvider
     from applitools.selenium.selenium_eyes import SeleniumEyes
 
 
@@ -51,8 +51,8 @@ class SeleniumVisualLocatorsProvider(VisualLocatorsProvider):
         data = VisualLocatorsData(
             app_name=self._eyes.configure.app_name,
             image_url=viewport_screenshot_url,
-            first_only=visual_locator_settings.is_first_only,
-            locator_names=visual_locator_settings.names,
+            first_only=visual_locator_settings.values.is_first_only,
+            locator_names=visual_locator_settings.values.names,
         )
         logger.info("Post visual locators: {}".format(data))
         return self._server_connector.post_locators(data)
