@@ -49,6 +49,8 @@ TRANSITIONS = [
     },
 ]
 
+END_OF_CHECKS = object()
+
 
 @attr.s(hash=False, str=False)
 class RunningTestCheck(object):
@@ -119,7 +121,8 @@ class RunningTestCheck(object):
                 "render_task_error", task_uuid=render_task.uuid, exc_info=e
             )
             self.running_test.pending_exceptions.append(e)
-            self.running_test.becomes_completed()
+            self.running_test.abort()
+            self.running_test.becomes_tested()
 
         render_task = VGTask(
             "RunningTest.render {} - {}".format(short_description, tag),
@@ -157,8 +160,7 @@ class RunningTestCheck(object):
                 self.running_test.task_lock = None
 
             self.running_test.watch_task[self] = True
-            if self.running_test.all_tasks_completed(self.running_test.watch_task):
-                self.running_test.becomes_tested()
+            self.running_test.maybe_becomes_tested()
 
         def check_task_error(e):
             self.logger.debug("check_task_error", task_uuid=check_task.uuid)
@@ -192,6 +194,8 @@ class RunningTest(object):
         # type: () -> None
         self.logger = self.logger.bind(running_test=self)
         self._initialize_vars()
+        self._is_closed = False
+        self._is_aborted = False
         self._initialize_state_machine()
         self.open()
 
@@ -233,11 +237,13 @@ class RunningTest(object):
             if self.task_lock:
                 return self.task_lock.queue
             elif self.task_queue:
-                self.task_lock = self.task_queue.popleft()
-                return self.task_lock.queue
-            elif self.close_queue:
-                # in case no checks, but close scheduled
-                return self.close_queue
+                item = self.task_queue.popleft()
+                if item is END_OF_CHECKS:
+                    # all checks are done and test is finished
+                    return self.close_queue
+                else:
+                    self.task_lock = item
+                    return self.task_lock.queue
             return deque()
         elif self.state == TESTED:
             return self.close_queue
@@ -260,7 +266,7 @@ class RunningTest(object):
             self.logger.debug("open_task_succeeded")
             self.watch_open[open_task] = True
             if self.all_tasks_completed(self.watch_open):
-                if self.state == TESTED:
+                if self.state in (TESTED, COMPLETED):
                     self.logger.debug("open_task_succeeded: test session was aborted")
                     return
                 self.becomes_opened()
@@ -297,6 +303,7 @@ class RunningTest(object):
 
     def close(self):
         # type: () -> Optional[Any]
+        self._is_closed = True
         if self.state == NEW:
             self.becomes_completed()
             return None
@@ -332,11 +339,20 @@ class RunningTest(object):
             close_task.on_task_error(close_task_error)
             self.close_queue.append(close_task)
             self.watch_close[close_task] = False
+            self.task_queue.append(END_OF_CHECKS)
+
+    @property
+    def is_closed(self):
+        return self._is_closed or self._is_aborted
 
     def abort(self):
-        # skip call of abort() in tests where close() already called
-        if self.close_queue or self.state == COMPLETED:
+        if self.state == COMPLETED:
             return None
+        self._is_aborted = True
+
+        while self.close_queue:
+            close_task = self.close_queue.pop()
+            self.watch_close.pop(close_task)
 
         def ensure_and_abort():
             if not self.eyes.is_open:
@@ -372,6 +388,7 @@ class RunningTest(object):
 
         self.close_queue.append(abort_task)
         self.watch_close[abort_task] = False
+        self.task_queue.append(END_OF_CHECKS)
 
     def all_tasks_completed(self, watch):
         # type: (Dict) -> bool
@@ -387,3 +404,11 @@ class RunningTest(object):
                 configuration_options or (), check_settings_options or ()
             )
         }
+
+    def maybe_becomes_tested(self):
+        if self.close_queue and self.all_tasks_completed(self.watch_task):
+            self.becomes_tested()
+
+    @property
+    def has_checks(self):
+        return bool(self.watch_task)
